<?php

namespace Drupal\book\Plugin\Validation\Constraint;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\book\BookManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Constraint validator for changing the book outline in pending revisions.
 */
class BookOutlineConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * Creates a new BookOutlineConstraintValidator instance.
   *
   * @param \Drupal\book\BookManagerInterface $bookManager
   *   The book manager.
   */
  public function __construct(protected BookManagerInterface $bookManager) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('book.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint): void {
    if (isset($value) && !$value->isNew() && !$value->isDefaultRevision()) {
      /** @var \Drupal\Core\Entity\ContentEntityInterface $original */
      $original = $this->bookManager->loadBookLink($value->id(), FALSE) ?: [
        'bid' => 0,
        'weight' => 0,
      ];
      if (empty($original['pid'])) {
        $original['pid'] = -1;
      }

      if ($value->book['bid'] != $original['bid']) {
        $this->context->buildViolation($constraint->message)
          ->atPath('book.bid')
          ->setInvalidValue($value)
          ->addViolation();
      }
      if ($value->book['pid'] != $original['pid']) {
        $this->context->buildViolation($constraint->message)
          ->atPath('book.pid')
          ->setInvalidValue($value)
          ->addViolation();
      }
      if ($value->book['weight'] != $original['weight']) {
        $this->context->buildViolation($constraint->message)
          ->atPath('book.weight')
          ->setInvalidValue($value)
          ->addViolation();
      }
    }
  }

}
